/*
* JBoss, Home of Professional Open Source.
* Copyright 2006, Red Hat Middleware LLC, and individual contributors
* as indicated by the @author tags. See the copyright.txt file in the
* distribution for a full listing of individual contributors. 
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/ 
package org.jboss.test.dependency.controller.test;

import org.jboss.dependency.plugins.AbstractDependencyItem;
import org.jboss.dependency.spi.ControllerContext;
import org.jboss.dependency.spi.ControllerState;
import org.jboss.test.dependency.controller.support.TestDelegate;

/**
 * 
 * @author <a href="kabir.khan@jboss.com">Kabir Khan</a>
 * @version $Revision: 1.1 $
 */
public class DependsOnMeTestCase extends AbstractDependencyTest
{
   public DependsOnMeTestCase(String name)
   {
      super(name);
   }

   public void testDependsOnMeCorrectOrder() throws Throwable
   {
      ControllerContext context1 = assertInstall(getDelegate1());
      assertEmpty(context1.getDependencyInfo().getIDependOn(null));
      assertEmpty(context1.getDependencyInfo().getDependsOnMe(null));
      
      ControllerContext context2 = assertInstall(getDelegate2());
      assertEmpty(context2.getDependencyInfo().getDependsOnMe(null));
      assertEquals(1, context2.getDependencyInfo().getIDependOn(null).size());
      assertEquals("Name1", context2.getDependencyInfo().getIDependOn(null).iterator().next().getIDependOn());
      
      assertEmpty(context1.getDependencyInfo().getIDependOn(null));
      assertEquals(1, context1.getDependencyInfo().getDependsOnMe(null).size());
      assertEquals("Name1", context1.getDependencyInfo().getDependsOnMe(null).iterator().next().getIDependOn());
   }
   
   public void testDependsOnMeWrongOrder() throws Throwable
   {
      ControllerContext context2 = assertInstall(getDelegate2(), ControllerState.PRE_INSTALL);
      assertEmpty(context2.getDependencyInfo().getDependsOnMe(null));
      assertEquals(1, context2.getDependencyInfo().getIDependOn(null).size());
      assertEquals("Name1", context2.getDependencyInfo().getIDependOn(null).iterator().next().getIDependOn());
      
      ControllerContext context1 = assertInstall(getDelegate1());
      assertEmpty(context1.getDependencyInfo().getIDependOn(null));
      assertEquals(1, context1.getDependencyInfo().getDependsOnMe(null).size());
      assertEquals("Name1", context1.getDependencyInfo().getDependsOnMe(null).iterator().next().getIDependOn());

      assertEmpty(context2.getDependencyInfo().getDependsOnMe(null));
      assertEquals(1, context2.getDependencyInfo().getIDependOn(null).size());
      assertEquals("Name1", context2.getDependencyInfo().getIDependOn(null).iterator().next().getIDependOn());
   }
   
   public void testDependsOnMeUninstallRightOrder() throws Throwable
   {
      ControllerContext context1 = assertInstall(getDelegate1());
      assertEmpty(context1.getDependencyInfo().getIDependOn(null));
      assertEmpty(context1.getDependencyInfo().getDependsOnMe(null));
      
      ControllerContext context2 = assertInstall(getDelegate2());
      assertEmpty(context2.getDependencyInfo().getDependsOnMe(null));
      assertEquals(1, context2.getDependencyInfo().getIDependOn(null).size());
      assertEquals("Name1", context2.getDependencyInfo().getIDependOn(null).iterator().next().getIDependOn());
      
      assertEmpty(context1.getDependencyInfo().getIDependOn(null));
      assertEquals(1, context1.getDependencyInfo().getDependsOnMe(null).size());
      assertEquals("Name1", context1.getDependencyInfo().getDependsOnMe(null).iterator().next().getIDependOn());
      
      assertUninstall(context2);
      assertEmpty(context2.getDependencyInfo().getDependsOnMe(null));
      assertEquals(1, context2.getDependencyInfo().getIDependOn(null).size());
      assertEquals("Name1", context2.getDependencyInfo().getIDependOn(null).iterator().next().getIDependOn());

      context1 = assertContext("Name1", ControllerState.INSTALLED);
      assertEmpty(context1.getDependencyInfo().getIDependOn(null));
      assertEmpty(context1.getDependencyInfo().getDependsOnMe(null));
   }
   
   
   protected TestDelegate getDelegate1()
   {
      return new TestDelegate("Name1");
   }
   
   protected TestDelegate getDelegate2()
   {
      TestDelegate result = new TestDelegate("Name2");
      result.addDependency(new AbstractDependencyItem("Name2", "Name1", ControllerState.DESCRIBED, ControllerState.INSTALLED));
      return result;
   }

}
